/**
 * Backward algoritmus implementacioja
 * @author Dombai Balazs
 */
public class Backward {
	
    private Hmm hmm;
    private Number[][][] matrix;
    private ReticularAlignment ra;

    private String seq1, seq2;
	
    /**
     * Konstruktor.
     * @param params (t/r/mu/lambda) parameterek
     * @param seq1 az elso szekvencia
     * @param seq2 a masodik szekvencia
     */
    public Backward(Params params, String seq1, String seq2) {
	this.seq1 = seq1;
	this.seq2 = seq2;
	ra = new ReticularAlignment(params.get_t());
	//	params.set_r(0.0); //mimicking the TKF91
	hmm = new Hmm(params);
	matrix = new Number[seq1.length() + 1][seq2.length() + 1][3];
		
	run();
    }
	
    /**
     * A teljes szekvencia emittalasi valoszinusegenek visszaadasa
     * @return valoszinuseg
     */
    public Number getProbability() {
	return new Number( matrix[0][0][0] );
    }

    public Number getPr(int i, int j, int k) {
    	return new Number(matrix[i][j][k]);
    }
	
    /**
     * A dinamikus programozasi algoritmus vegrehajtasa.
     */
    private void run() {
    	for (int i = 0; i < 3; i++) {
	    matrix[seq1.length()][seq2.length()][i] = new Number(hmm.valueAt(i,3));
    	}
	for (int j = seq1.length() - 1; j >= 0; j--) {
	    char c1 = seq1.charAt(j);
	    for(int i = 0; i < 3; i++){
		matrix[j][seq2.length()][i] = new Number(matrix[j+1][seq2.length()][1]);
		matrix[j][seq2.length()][i].mul(hmm.valueAt(i,1));
		matrix[j][seq2.length()][i].mul(ra.getPi(c1));
	    }
	}
	for (int k = seq2.length() - 1; k >= 0; k--) {
	    char c2 = seq2.charAt(k);
	    for(int i = 0; i < 3; i++){
		matrix[seq1.length()][k][i] = new Number(matrix[seq1.length()][k+1][2]);
		matrix[seq1.length()][k][i].mul(hmm.valueAt(i,2));
		matrix[seq1.length()][k][i].mul(ra.getPi(c2));
	    }
	}
    	
    	for (int j = seq1.length()-1; j >= 0; j--) {
	    for (int k = seq2.length()-1; k >= 0; k--) {
		matrix[j][k][0] = new Number(0);
		matrix[j][k][1] = new Number(0);
		matrix[j][k][2] = new Number(0);

		char c1 = seq1.charAt(j);
		char c2 = seq2.charAt(k);
		
		for (int i = 0; i < 3; i++) {    				
		    Number tmp = new Number(matrix[j+1][k+1][0]);
		    tmp.mul( ra.getPi(c1) * ra.getTrans(c2, c1) * hmm.valueAt(i,0));
		    matrix[j][k][i].add(tmp);
		}

		for (int i = 0; i < 3; i++) {
		    Number tmp = new Number(matrix[j+1][k][1]);
		    tmp.mul( ra.getPi(c1) * hmm.valueAt(i,1) );
		    matrix[j][k][i].add(tmp);
		}

		for (int i = 0; i < 3; i++) {
		    Number tmp = new Number(matrix[j][k+1][2]);
		    tmp.mul(  ra.getPi(c2) * hmm.valueAt(i,2) );
		    matrix[j][k][i].add(tmp);
		}

	    }
    	}
    }

}
