import java.io.*;

/**
 * Blosum-matrix beolvasasa file-bol es kezelese
 * @author Dombai Balazs
 */
public class Blosum {
	
    /**
     * A matrix alapertelmezett merete. 
     */
    static final int MATRIX_SIZE = 24;
    private int[][] matrix;
    private String[] labels;
    public int type;
	
    /** 
     * Konstruktor, az alapertelmezett BLOSUM62 erteket hasznalja. 
     */
    public Blosum() {
	loadMatrix(BLOSUM62);
	type = 62;
    }
	
    /**
     * Konstruktor, a megadott BLOSUMxxx erteket hasznalja.
     * @param type a matrix tipusa
     */
    public Blosum(int type) {
	loadMatrix(type);
	this.type = type;
    }
	
    /**
     * Betolti a megadott Blosum-matrixot a megfelelo file-bol.
     * @param type melyik matrixot akarjuk betolteni
     */
    private void loadMatrix(int type) {
	matrix = new int[MATRIX_SIZE][MATRIX_SIZE];
	try {
	    BufferedReader infile = new BufferedReader( new FileReader("blosum" + type + ".bla") );

	    String buffer = infile.readLine();

	    // Komment sorok olvasasa, nincs rajuk szukseg
	    while ( buffer.startsWith("#") ) {
		buffer = infile.readLine();
	    }
			
	    // Az aminosav-cimkek beolvasasa es feldolgozasa
	    labels = buffer.trim().split(" +");
			
	    // A matrix elemeinek feldolgozasa
	    for (int i = 0; i < MATRIX_SIZE; i++) {
		String[] tmp = infile.readLine().trim().split(" +");
		for (int j = 0; j < MATRIX_SIZE; j++) {
		    matrix[i][j] = Integer.parseInt(tmp[j]);
		}
	    }
	    infile.close();
	} catch (Exception e) {
	    System.err.println( e.getMessage() );
	}
    }

    /**
     * Matrix adott elemenek lekerdezese (int koordinatak)
     * @param x x-koordinata
     * @param y y-koordinata
     * @return a keresett elem
     */
    public int elementAt(int x, int y) {
	return matrix[x][y];
    }
	
    /**
     * Matrix adott elemenek lekerdezese (char koordinatak)
     * @param x x-koordinata
     * @param y y-koordinata
     * @return a keresett elem
     */
    public int elementAt(char x, char y) {
	int xx = MATRIX_SIZE - 1, yy = MATRIX_SIZE - 1;
	for (int i = 0; i < MATRIX_SIZE; i++) {
	    if ( labels[i].charAt(0) == x ) {
		xx = i;
		break;
	    }
	}
	for (int j = 0; j < MATRIX_SIZE; j++) {
	    if ( labels[j].charAt(0) == y ) {
		yy = j;
		break;
	    }
	}
	return matrix[xx][yy];
    }
	
    /** BLOSUM35 */
    static final int BLOSUM35 = 35;
    /** BLOSUM40 */
    static final int BLOSUM40 = 40;
    /** BLOSUM45 */
    static final int BLOSUM45 = 45;
    /** BLOSUM50 */
    static final int BLOSUM50 = 50;
    /** BLOSUM62 */
    static final int BLOSUM62 = 62;
    /** BLOSUM80 */
    static final int BLOSUM80 = 80;
    /** BLOSUM100 */
    static final int BLOSUM100 = 100;
	
}
