/**
 * Konjugalt gradiens modszer implementacioja
 * @author Dombai Balazs
 * Jelentosen atirva, 2006 november: Miklos Istvan
 */

public class ConjGrad {
	
    private Params params;
    private Number value;
    private int numsteps = 0;
	
    private String seq1, seq2;
	
    private double jump;
    private boolean end = false;
    private double eps;
    private int lastDir = 0;
	
    private Number[] oldGradient = new Number[] {new Number(1.0), new Number(1.0), new Number(1.0)};
    

    /*
     * Konstruktor.
     */
    public ConjGrad(Params params, String seq1, String seq2, int eps) {
	this.params = params;
	this.seq1 = seq1;
	this.seq2 = seq2;
	this.eps = Math.pow(10, -eps);
	//jump = this.eps;
	jump = 0.02;
	value = new Forward(params, seq1, seq2).getProbability();
    }
	
    public Params getOptimizedParams() {
	return params;
    }
	
    public Number getValue() {
	return value;
    }
	
    public boolean end() {
	return end;
    }
	
    public void step() {
	//	maxjump = Math.max( eps, maxjump * ((-1 + Math.sqrt(5)) / (1 + Math.sqrt(5))) );
		
	Number oldProbability = new Forward(params, seq1, seq2).getProbability();
	Number[] newGradient = new Number[3];
	double max = 0.0;
	
	// Kiszamoljuk a gradienst
	Number norm = new Number(0.0);
	oldProbability.alap = -oldProbability.alap;
	for (int j = 0; j < 3; j++) { 
	    Params params_tmp = new Params(params, seq1.length() + seq2.length() - 2);
	    params_tmp.addToValue(j, eps);
	    newGradient[j] = new Number(new Forward(params_tmp, seq1, seq2).getProbability());
	    newGradient[j].add(oldProbability);
	    newGradient[j].mul(1.0/eps);
	    Number temp = new Number(newGradient[j]);
	    temp.mul(newGradient[j]);
	    norm.add(temp);
	}
	System.out.println("Gradient before normalization: "+newGradient[0].toString()+" "+newGradient[1].toString()+" "+newGradient[2].toString());
	System.out.println("Norma: "+norm.toString());
	norm.exponent = -norm.exponent;
	norm.sqrt();
	for (int j = 0; j < 3; j++) { 
	    newGradient[j].mul(norm);
	}
	System.out.println("Gradient: "+newGradient[0].toString()+" "+newGradient[1].toString()+" "+newGradient[2].toString());

	//modositjuk az ertekeket
	Params newParams = new Params(params, seq1.length() + seq2.length() - 2);
	for(int j = 0; j < 3; j++){
	    Number temp = new Number(newGradient[j]);
	    temp.mul(jump);
	    newParams.addToValue(j,temp.toDouble());
	}
	Number newProbability = new Forward(newParams, seq1, seq2).getProbability();
	oldProbability.alap = - oldProbability.alap;
	if(Number.greater(newProbability,oldProbability)){
	    value = newProbability;
	    //lepunk a gradiens iranyaba...
	    for(int j = 0; j < 3; j++){
		Number temp = new Number(newGradient[j]);
		temp.mul(jump);
		params.addToValue(j,temp.toDouble());
	    }
	    //megnoveljuk a jump erteket
	    jump *= ((1.0 + Math.sqrt(5.0)) / (-1.0 + Math.sqrt(5.0)));
	    
	}
	else{
	    while(!Number.greater(newProbability,oldProbability) && jump >= eps){
		jump *= ((-1 + Math.sqrt(5)) / (1 + Math.sqrt(5)));
		newParams = new Params(params, seq1.length() + seq2.length() - 2);
		for(int j = 0; j < 3; j++){
		    Number temp = new Number(newGradient[j]);
		    temp.mul(jump);
		    newParams.addToValue(j,temp.toDouble());
		}
		newProbability = new Forward(newParams, seq1, seq2).getProbability();
	    }
	    if(jump < eps){
		end = true;
	    }
	    else{
		value = newProbability;
		for(int j = 0; j < 3; j++){
		    Number temp = new Number(newGradient[j]);
		    temp.mul(jump);
		    params.addToValue(j,temp.toDouble());
		}
	    }
	}
	
// 	if (direction == 0) { // megvan a maximum, nem kell folytatni
// 	    end = true;
// 	} else {
// 	    lastDir = direction;
// 	    boolean continuing = true;
// 	    double jump = maxjump;
			
// 	    while (continuing) { // iteracio az adott irany menten
// 		Params newparams = new Params(params, seq1.length() + seq2.length() - 2);
// 		newparams.addToValue(Math.abs(direction), (direction > 0 ? 1 : -1) * jump);

// 		Number newvalue = new Forward(newparams, seq1, seq2).getProbability();
				
// 		if (Number.greater(value, newvalue) || Number.equals(value, newvalue)) {
// 		    if (jump <= eps) {
// 			continuing = false;
// 		    } else {
// 			jump *= ((-1 + Math.sqrt(5)) / (1 + Math.sqrt(5)));
// 		    }
// 		} else {
// 		    value = newvalue;
// 		    params = newparams;
// 		    jump *= ((1 + Math.sqrt(5)) / (-1 + Math.sqrt(5)));
// 		    maxjump = Math.max(jump, maxjump);
// 		}
// 	    }
// 	}
		
    }
	
}
