import java.text.*;
import java.awt.*;
import java.awt.font.*;
import java.awt.geom.*;
import javax.swing.*;

/**
 * DiagramPanel osztaly
 * @author Dombai Balazs
 */
public class DiagramPanel extends JPanel {
	
	private ResultSet rs = null;
	private String title;
	private int numCols;
	private int numRows;
	
	static final int COLUMN_WIDTH = 9;
	static final int FONT_HEIGHT = 11;
	static final int OFFSET_X = 10;
	static final int OFFSET_Y = 10;
	static final int TITLE_Y = 20;
	
	public DiagramPanel() {
	}
	
	public void updateData(ResultSet rs, String title) {
		this.rs = rs;
		this.title = title;
		numCols = rs.getMaxPos() + 1;
		numRows = rs.getName(0).length();
	}
	
	public void paintComponent(Graphics gr) {
		super.paintComponent(gr);
		Graphics2D g = (Graphics2D)gr.create();
		
		g.setBackground(Color.WHITE);
		g.clearRect(0, 0, this.getWidth(), this.getHeight());
		
		if (rs != null) {
			int colHeight = g.getClipBounds().height - (2 * OFFSET_Y + TITLE_Y + numRows * FONT_HEIGHT);
			double max = 0;
			for (int i = 0; i < numCols; i++) {
				max = Math.max(max, rs.getValue(i));
			}

			g.setColor(Color.LIGHT_GRAY);
			for (int i = 0; i < 5; i++) {
				g.drawLine( OFFSET_X,
					OFFSET_Y + TITLE_Y + Math.round(i * (float)colHeight / 4),
					OFFSET_X + numCols * COLUMN_WIDTH,
					OFFSET_Y + TITLE_Y + Math.round(i * (float)colHeight / 4));
			}
			
			g.setColor(Color.BLACK);
			g.setFont(new Font("SANS_SERIF", Font.BOLD, 16));
			g.drawString(title, OFFSET_X, TITLE_Y);
			g.setFont(new Font("MONOSPACED", Font.PLAIN, 12));
			g.setClip(0,0, COLUMN_WIDTH * numCols + 2 * OFFSET_X, g.getClipBounds().height);			

			for (int i = 0; i < numCols; i++) {
				g.setColor(Color.BLACK);
				for (int j = 0; j < rs.getName(i).length(); j++) {
					g.drawString(rs.getName(i).charAt(j) + "",
						OFFSET_X + COLUMN_WIDTH * i,
						colHeight + OFFSET_Y + TITLE_Y + FONT_HEIGHT * (j+1));
				}
				
				g.setColor(Color.BLUE);
				g.drawLine( OFFSET_X + i * COLUMN_WIDTH + COLUMN_WIDTH / 2,
					(int)(OFFSET_Y + TITLE_Y + colHeight - Math.round(rs.getValue(i) * (colHeight) / max)),
					OFFSET_X + i * COLUMN_WIDTH + COLUMN_WIDTH / 2,
					OFFSET_Y + TITLE_Y + colHeight);
			}
		}
	}
	
	public Dimension getMinimumSize(){
		return getPreferredSize();
	}
	
	public Dimension getPreferredSize() {
		if (rs != null) {
			return new Dimension((rs.getMaxPos() + 3) * COLUMN_WIDTH, 100);
		} else {
			return new Dimension(0,100);
		}
	}
	
}
