/**
 * Forward algoritmus implementacioja
 * @author Dombai Balazs
 */
public class Forward {
	
    private Hmm hmm;
    private Number[][][] matrix;
    private ReticularAlignment ra;

    private String seq1, seq2;
	
    /**
     * Konstruktor.
     * @param params (t/r/mu/lambda) parameterek
     * @param seq1 az elso szekvencia
     * @param seq2 a masodik szekvencia
     */
    public Forward(Params params, String seq1, String seq2) {
	this.seq1 = '-' + seq1;
	this.seq2 = '-' + seq2;
	ra = new ReticularAlignment(params.get_t());
	//params.set_r(0.0); //mimicking the TKF91...
	hmm = new Hmm(params);
	matrix = new Number[this.seq1.length()][this.seq2.length()][3];

	run();
    }
	
    /**
     * A teljes szekvencia emittalasi valoszinusegenek visszaadasa
     * @return valoszinuseg
     */
    public Number getProbability() {
    	Number tmp = new Number(0);
    	for (int i = 0; i < 3; i++) {
	    Number tmp1 = new Number(matrix[seq1.length()-1][seq2.length()-1][i]);
	    tmp1.mul(hmm.valueAt(i,3));
	    tmp.add(tmp1);
    	}
    	return tmp;
    }
	
    public Number getPr(int i, int j, int k) {
	return new Number(matrix[i][j][k]);
    }
	
    /**
     * A dinamikus programozasi algoritmus vegrehajtasa.
     */
    private void run() {
	matrix[0][0][0] = new Number( 1 );
 	matrix[0][0][1] = new Number( 0 );
 	matrix[0][0][2] = new Number( 0 );
 	for (int j = 1; j < seq1.length(); j++) {
	    matrix[j][0][0] = new Number(0);

	    matrix[j][0][1] = new Number(0);
	    Number temp = new Number(matrix[j-1][0][0]);
	    char c1 = seq1.charAt(j);
	    temp.mul(ra.getPi(c1) * hmm.valueAt(0,1));
	    matrix[j][0][1].add(temp);
	    temp = new Number(matrix[j-1][0][1]);
	    temp.mul(ra.getPi(c1) * hmm.valueAt(1,1));
	    matrix[j][0][1].add(temp);	    

	    matrix[j][0][2] = new Number(0);
	    
	}
	for (int k = 1; k < seq2.length(); k++) {
	    matrix[0][k][0] = new Number(0);
	    matrix[0][k][1] = new Number(0);

	    matrix[0][k][2] = new Number(0);
	    Number temp = new Number(matrix[0][k-1][0]);
	    char c2 = seq2.charAt(k);
	    temp.mul(ra.getPi(c2) * hmm.valueAt(0,2));
	    matrix[0][k][2].add(temp);
	    temp = new Number(matrix[0][k-1][2]);
	    // Istvan: old line was: temp.mul(ra.getPi(c2) * hmm.valueAt(2,1));
	    // new:
	    temp.mul(ra.getPi(c2) * hmm.valueAt(2,2));
	    matrix[0][k][2].add(temp);	    
	}

    	for (int j = 1; j < seq1.length(); j++) {
	    for (int k = 1; k < seq2.length(); k++) {
		matrix[j][k][0] = new Number(0);
		matrix[j][k][1] = new Number(0);
		matrix[j][k][2] = new Number(0);

		char c1 = seq1.charAt(j);
		char c2 = seq2.charAt(k);		

		for (int i = 0; i < 3; i++) {    				
		    Number tmp = new Number(matrix[j-1][k-1][i]);
		    tmp.mul( ra.getPi(c1) * ra.getTrans(c2, c1) * hmm.valueAt(i,0));
		    matrix[j][k][0].add(tmp);
		}

		for (int i = 0; i < 3; i++) {
		    Number tmp = new Number(matrix[j-1][k][i]);
		    tmp.mul( ra.getPi(c1) * hmm.valueAt(i,1) );
		    matrix[j][k][1].add(tmp);
		}

		for (int i = 0; i < 3; i++) {
		    Number tmp = new Number(matrix[j][k-1][i]);
		    tmp.mul(  ra.getPi(c2) * hmm.valueAt(i,2) );
		    matrix[j][k][2].add(tmp);
		}

	    }
    	}

    }

}
