/**
 * A Gotoh-algoritmus implementacioja, a nyitasi buntetes -10, a kiterjesztesi
 * buntetes -1.
 * @author Dombai Balazs
 */
public class GotohPenalty implements IPenalty {
	
    private int[][] d, d2; // ertek- es iranymatrix
    private String str1, str2;
    private SimilarityScore scoreMatrix;
	
    private int[] max_p; // oszlopokban levo maximum
    private int max_q; // adott sorban levo maximum

    private int max1, max2; // max. elem koordinatai
	
    /**
     * Konstruktor.
     * @param blosum a hasznalt Blosum-matrix
     */
    public GotohPenalty(SimilarityScore scoreMatrix) {
	this.scoreMatrix = scoreMatrix;
    }

    public GotohPenalty(Blosum blosum){
	scoreMatrix = new SimilarityScore(blosum.type);
    }	
    /**
     * Inicializalas, d elso soranak es oszlopanak kiszamitasa.
     * @param str1 elso szekvencia
     * @param str2 masodik szekvencia
     */
    public void init(String str1, String str2) {
	this.str1 = str1;
	this.str2 = str2;
	d = new int[str1.length() + 1][str2.length() + 1];
	d2 = new int[str1.length() + 1][str2.length() + 1];
		
	max_p = new int[str2.length() + 1];
	for (int i = 0; i <= str2.length(); i++) {
	    max_p[i] = 0;
	}
		
	d[0][0] = 0;
	d2[0][0] = STOP;
	for (int i = 1; i <= str1.length(); i++) {
	    d[i][0] = 0;
	    d2[i][0] = STOP;
	}
	for (int j = 1; j <= str2.length(); j++) {
	    d[0][j] =0;
	    d2[0][j] =STOP;
	}
    }
	
    /**
     * A d tobbi ertekenek kiszamolasa.
     */
    public void run() {
	for (int i = 1; i <= str1.length(); i++) {
	    max_q = 0;
			
	    for (int j = 1; j <= str2.length(); j++) {
		int dist1 = d[i - 1][j - 1] + scoreMatrix.elementAt(str1.charAt(i - 1), str2.charAt(j - 1));
		int dist2 = p(i,j);
		int dist3 = q(i,j);
				
		d[i][j] = Math.max(
				   Math.max(0, dist1),
				   Math.max(dist2, dist3)
				   );
				
		if (d[i][j] == dist1) d2[i][j] = UPLEFT;
		else if (d[i][j] == dist2) d2[i][j] = UP;
		else if (d[i][j] == dist3) d2[i][j] = LEFT;
		else d2[i][j] = STOP;
	    }
	}
    }
	
    /**
     * Maximalis elem keresese a matrixban.
     * @return maximalis elem
     */
    public int findMax() {
	int max = d[0][0];
	max1 = max2 = 0;
	for (int i = 0; i <= str1.length(); i++) {
	    for (int j = 0; j <= str2.length(); j++) {
		if ( d[i][j] >= max ) {
		    max = d[i][j];
		    max1 = i;
		    max2 = j;
		}
	    }
	}
	return max;
    }
	
    public int get_max1() {
	return max1;
    }
	
    public int get_max2() {
	return max2;
    }
	
    /**
     * Visszaadja a Gotoh-algoritmus altal kiszamolt (m+1)*(n+1)-es matrixot.
     * @return matrix
     */
    public int[][] getMatrix() {
	return d;
    }
	
    /**
     * Az iranymatrix visszaadasa, ebbol kaphatjuk meg az optimalis illesztest.
     * @return iranymatrix
     */
    public int[][] getDirectionMatrix() {
	return d2;
    }
	
    /**
     * P(i,j) segedfuggveny.
     * @param i i
     * @param j j
     * @return P(i,j)
     */
    private int p(int i, int j) {
	if (i == 0) return 0;
	max_p[j] = Math.max( max_p[j] + EXTENSION_PENALTY, d[i-1][j]+OPENING_PENALTY );
	return max_p[j];
    }
	
    /**
     * Q(i,j) segedfuggveny.
     * @param i i
     * @param j j
     * @return Q(i,j)
     */
    private int q(int i, int j) {
	if (j == 0) return 0;
	max_q = Math.max(max_q + EXTENSION_PENALTY, d[i][j-1]+OPENING_PENALTY);
	return max_q;
    }
	
    /** Nyitasi buntetes. */
    static int OPENING_PENALTY = -10;
    /** Kiterjesztesi buntetes. */
    static int EXTENSION_PENALTY = -1;
	
}
