/**
 * Rejtett Markov-modell.
 * @author Dombai Balazs
 */
public class Hmm {
	
    private Params params;
	
    /** 
     * A rejtett Markov-modell atmenetmatrixa.
     * 0 MAT
     * 1 DEL
     * 2 INS
     * 3 END
     */
    private double[][] m;
	
    /**
     * Atmenetmatrix adott elemenek lekerdezese.
     * @param from honnan
     * @param to hova
     * @return a keresett elem
     */
    public double valueAt(int from, int to) {
	//System.out.println("From: "+from+" to: "+to);
	return m[from + 1][to + 1];
    }

    /** 
     * Konstruktor.
     * @param params kezdeti parameterek
     */
    public Hmm(Params params) {
	this.params = params;
	m = new double[5][5];
	for (int i = 0; i < 5; i++) {
	    for (int j = 0; j < 5; j++) {
		m[i][j] = 0.0;
	    }
	}
	calculate_m();
    }
	
    /**
     * m matrix kiszamitasa.
     */
    private void calculate_m() {
	double tmp = params.get_lambda() / params.get_mu();
	double tmp2 = 1 - params.get_r() - get_beta() + params.get_r() * get_beta();
	    
	m[0][1] = (1 - get_beta()) * tmp * get_alpha();
	m[0][2] = (1 - get_beta()) * tmp * (1 - get_alpha());
	m[0][3] = get_beta();
	m[0][4] = (1 - get_beta()) * (1 - tmp);
		
	m[1][1] = params.get_r() + (1 - params.get_r()) * (1 - get_beta()) * tmp * get_alpha();
	m[1][2] = (1 - params.get_r()) * (1 - get_beta()) * tmp * (1 - get_alpha());
	m[1][3] = (1 - params.get_r()) * get_beta();
	m[1][4] = (1 - params.get_r()) * (1 - get_beta()) * (1 - tmp);
		
	m[2][1] = (1 - params.get_r()) * (1 - get_gamma()) * tmp * get_alpha();
	m[2][2] = params.get_r() + (1 - params.get_r()) * (1 - get_gamma()) * tmp * (1 - get_alpha());
	m[2][3] = (1 - params.get_r()) * get_gamma();
	m[2][4] = (1 - params.get_r()) * (1 - get_gamma()) * (1 - tmp);
		
	m[3][1] = tmp2 * tmp * get_alpha();
	m[3][2] = tmp2 * tmp * (1 - get_alpha());
	m[3][3] = 1 - tmp2;
	m[3][4] = tmp2 * (1 - tmp);
    }
	
    /**
     * Alfa parameter kiszamolasa.
     * @return alfa
     */
    public double get_alpha() {
	return Math.exp( -params.get_mu() * params.get_t() );
    }
	
    /**
     * Beta parameter kiszamitasa.
     * @return beta
     */
    public double get_beta() {
	double szamlalo = params.get_lambda() *	( 1 - Math.exp(	params.get_t()*(params.get_lambda()-params.get_mu()) ) );
	double nevezo = params.get_mu() - params.get_lambda() * ( Math.exp( params.get_t()*(params.get_lambda()-params.get_mu()) ) );
		
	return szamlalo / nevezo;
    }
	
    /**
     * Gamma parameter kiszamitasa.
     * @return gamma
     */
    public double get_gamma() {
	double szamlalo = params.get_mu() *
	    ( 1 - Math.exp(
			   params.get_t()*(params.get_lambda()-params.get_mu())
			   ) );
	double nevezo = ( 1 - Math.exp(-params.get_mu() * params.get_t()) ) *
	    ( params.get_mu() - params.get_lambda() * ( Math.exp( params.get_t()*(params.get_lambda()-params.get_mu()) ) ) );
	return 1 - szamlalo / nevezo;
    }
	
    /**
     * Returns a string representation of the object.
     * @return a string representation of the object
     */
    public String toString() {
	// Istvan: for debugging reasons 
	String temp = "";
	for(int i = 1; i < 4; i++){
	    for(int j = 1; j < 4; j++){
		temp = temp + "\t" + m[i][j];
	    }
	    temp = temp + "\n";
	}
	return temp+ get_alpha() +" "+ get_beta() +" "+ get_gamma();
    }
	
}
