import javax.swing.*;
import javax.swing.border.*;
import javax.swing.filechooser.*;
import java.awt.*;
import java.awt.event.*;

/**
 * A program foablaka
 * @author Dombai Balazs
 */
public class MainFrame extends JFrame implements ActionListener {
    
    private PatternSet ps = null;
    private Settings settings = null;
    private SimilarSequences simSeq = null;
    
    public MainFrame() {
	super();
	c = getContentPane();
	
	settings = new Settings();
	
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	setLayout( new BorderLayout() );
	setPreferredSize( new Dimension(600, 400) );
	setTitle(WINDOW_TITLE);
	
	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		if (JOptionPane.YES_OPTION != JOptionPane.showConfirmDialog(e.getWindow(), "Are you sure?", "Confirmation", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)) {
		    setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		} else {
		    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		}
	    }
	});
	
	initMenuBar();
	initComponents();
    } // MainFrame()
    
    private void openItem_clicked() {
	JFileChooser fc = new JFileChooser();
	fc.setCurrentDirectory(new java.io.File("."));
	
	fc.addChoosableFileFilter(new FileFilter() {
	    public String getDescription() { return "Fasta files (*.txt, *.fasta)"; }
	    public boolean accept(java.io.File f) {
		if (f.isDirectory()) {
		    return true;
		}
		if (f.getAbsolutePath().endsWith(".txt") || f.getAbsolutePath().endsWith(".fasta")) {
		    return true;
		}
		return false;
	    }
	});
	
	if (JFileChooser.APPROVE_OPTION == fc.showOpenDialog(this)) {
	    ps = new PatternSet(fc.getSelectedFile().toString());
	    preprocessItem.setEnabled(true);
	    runItem.setEnabled(false);
	    setTitle(WINDOW_TITLE + " - " + fc.getSelectedFile().toString());
	} else {
	    preprocessItem.setEnabled(false);
	    runItem.setEnabled(false);
	}
    } // openItem_clicked()
    
    private void exitItem_clicked() {
	if (JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(this, "Are you sure?", "Confirmation", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)) {
	    dispose();
	}
    } // exitItem_clicked()
    
    private void settingsItem_clicked() {
	SettingsDialog sd = new SettingsDialog(this, settings);
	sd.setVisible(true);
	if (sd.ok_clicked) {
	    settings = sd.getSettings();
	}
	sd.dispose();
    } // settingsItem_clicked()
    
    private void preprocessItem_clicked() {
	PreprocessDialog pd = new PreprocessDialog(this, ps, settings);
	pd.setVisible(true);
	if (pd.ok_clicked) {
	    runItem.setEnabled(true);
	    
	    simSeq = pd.getResult();
//	    seqIndex1 = pd.getSeqIndex1();
//	    seqIndex2 = pd.getSeqIndex2();
	} else {
	    runItem.setEnabled(false);
	}
	pd.dispose();
    } // preprocessItem_clicked()
    
    private void runItem_clicked() {
	String titleNow = getTitle();
//	RunDialog rd = new RunDialog(this, settings, ps.elementAt(seqIndex1).getValue(),  ps.elementAt(seqIndex2).getValue());
	RunDialog rd = new RunDialog(this, settings, simSeq);
	rd.setVisible(true);
	if (rd.abort_clicked) {
	    JOptionPane.showMessageDialog(this, "Process aborted.", "Warning", JOptionPane.WARNING_MESSAGE);
//	    setTitle(titleNow);
	} else {
	    diagramPanel.updateData(rd.getResultSet(), simSeq.getName());
	    diagramPanel.updateUI();
	}
	setTitle(titleNow);
	rd.dispose();
//	}
    } // runItem_clicked()
    
    private void initMenuBar() {
	// File menu
	(openItem = new JMenuItem("Open...")).addActionListener(this);
	openItem.setMnemonic('O');
	(exitItem = new JMenuItem("Exit")).addActionListener(this);
	exitItem.setMnemonic('E');
	fileMenu = new JMenu("File");
	fileMenu.setMnemonic('F');
	fileMenu.add(openItem);
	fileMenu.add( new javax.swing.JSeparator() );
	fileMenu.add(exitItem);
	// Run menu
	(preprocessItem = new JMenuItem("Preprocess...")).addActionListener(this);
	preprocessItem.setMnemonic('P');
	preprocessItem.setEnabled(false);
	(runItem = new JMenuItem("Run...")).addActionListener(this);
	runItem.setMnemonic('u');
	runItem.setEnabled(false);
	(settingsItem = new JMenuItem("Settings...")).addActionListener(this);
	settingsItem.setMnemonic('S');
	runMenu = new JMenu("Run");
	runMenu.setMnemonic('R');
	runMenu.add(preprocessItem);
	runMenu.add(runItem);
	runMenu.add( new javax.swing.JSeparator() );
	runMenu.add(settingsItem);
	// menu
	menu = new JMenuBar();
	menu.add(fileMenu);
	menu.add(runMenu);
	this.setJMenuBar(menu);
    } // initMenuBar()
    
    private void initComponents() {
	diagramPanel = new DiagramPanel();
	c.add(new JScrollPane(diagramPanel), BorderLayout.CENTER);
	//status bar
	statusText = new JLabel("Phylogeny Cafe :: StructureDecoder");
	statusBar = new JPanel();
	statusBar.setLayout(new BorderLayout());
	statusBar.setBorder(new BevelBorder(BevelBorder.LOWERED));
	statusBar.add(statusText, BorderLayout.CENTER);
	c.add(statusBar, BorderLayout.SOUTH);
	
	pack();
    } // initComponents()
    
    public void actionPerformed(ActionEvent evt) {
	if (evt.getSource() == openItem) {
	    openItem_clicked();
	} else if (evt.getSource() == exitItem) {
	    exitItem_clicked();
	} else if (evt.getSource() == preprocessItem) {
	    preprocessItem_clicked();
	} else if (evt.getSource() == runItem) {
	    runItem_clicked();
	} else if (evt.getSource() == settingsItem) {
	    settingsItem_clicked();
	}
    } // actionPerformed(evt)
    
    static final String WINDOW_TITLE = "Window title";
    
    private Container c;
    // components
    private JMenuBar menu;
    private JMenu fileMenu;
    private JMenuItem openItem, exitItem;
    private JMenu runMenu;
    private JMenuItem preprocessItem, runItem, settingsItem;
    
    private JPanel statusBar;
    private JLabel statusText;
    private DiagramPanel diagramPanel;
}
