/**[21~
 * Sajat szamabrazolas megvalositasa
 * @author Dombai Balazs
 */
public class Number {
    
    public double alap;
    public long exponent;
    
    /*
     * Alapertelmezett konstruktor.
     */
    Number() {
	alap = 0.0;
	exponent = 0;
    }
    
    /*
     * Konstruktor valos kezdetiertekkel.
     */
    Number(double n) {
	alap = n;
	exponent = 0;
	optimize();
    }
    
    /*
     * Konstruktor normalalakkal.
     */
    Number(double alap, long exponent) {
	this.alap = alap;
	this.exponent = exponent;
	optimize();
    }
    
    /*
     * Konstruktor Number tipussal.
     */
    Number(Number n) {
	alap = n.alap;
	exponent = n.exponent;
    }
    
    /*
     * Szorzas.
     * @param n Number tipusu szorzo
     */
    public void mul(Number n) {
	alap *= n.alap;
	exponent += n.exponent;
	optimize();
    }
    
    /*
     * Szorzas.
     * @param n valos tipusu szorzo
     */
    public void mul(double n) {
	alap *= n;
	optimize();
    }
    
    /*
     * Osszadas.
     * @param n Number tipusu szam
     */
    public void add(Number n) {
	if (alap == 0 || n.alap == 0) {
	    exponent = (alap == 0 ? n.exponent : exponent);
	    alap += n.alap;
	    return;
	} else if (n.exponent < exponent) {
	    alap += n.alap / Math.pow(10, exponent - n.exponent);
	    optimize();
	    return;
	} else {
	    alap = alap / Math.pow(10, n.exponent - exponent) + n.alap;
	    exponent = n.exponent;
	    optimize();
	    return;
	}
    }


    /*
     *gyokvonas
     *
     */    

    public void sqrt(){
	if(alap < 0){
	    throw new Error("You want to take the root of a negative number!!! "+this.toString());
	}
	else{
	    if(exponent % 2 == 1){
		alap *= 10.0;
		exponent--;
	    }
	    alap = Math.sqrt(alap);
	    exponent /= 2;
	    optimize();

	}
    }

    /*
     * Normalizalas
     */
    public void optimize() {
	if (Double.isNaN(alap)) {
	    throw new Error("Az alap nem szam!!!");
	}
	if (alap > 0) {
	    while (alap < 1 || alap >= 10) {
		if (alap < 1) {
		    alap *= 10;
		    exponent--;
		} else if (alap >= 10) {
		    alap /= 10;
		    exponent++;
		}
	    }
	}
    }
    
    /*
     * Egyenlosegvizsgalat.
     * @param n1 elso szam
     * @param n2 masodik szam
     * @return igaz ha n1=n2, hamis kulonben
     */
    public static boolean equals(Number n1, Number n2) {
	if (n1.alap == n2.alap && n1.alap == 0) {
	    return true;
	}
	if (n1.alap == n2.alap && n1.exponent == n2.exponent) {
	    return true;
	} else {
	    return false;
	}
    }
    
    /* "nagyobb mint" relacio.
     * @param n1 elso szam
     * @param n2 masodik szam
     * @return igaz ha n1>n2, hamis kulonben
     */
    public static boolean greater(final Number n1, final Number n2) {
	if (n1.alap == 0) {
	    return false;
	} else if (n2.alap == 0) {
	    return true;
	}
	
	if (n1.exponent < n2.exponent) {
	    return false;
	} else if (n1.exponent > n2.exponent) {
	    return true;
	}
	
	if (n1.alap < n2.alap) {
	    return false;
	}
	
	return true;
    }
    
    /*
     * Maximumkivalasztas ket szam kozul.
     * @param n1 elso szam
     * @param n2 masodik szam
     * @return max(n1,n2)
     */
    public static Number max(Number n1, Number n2) {
	if (greater(n1, n2)) {
	    return new Number(n1);
	} else {
	    return new Number(n2);
	}
    }
    
    /*
     * Maximumkivalasztas harom szam kozul.
     * @param n1 elso szam
     * @param n2 masodik szam
     * @param n2 harmadik szam
     * @return max(n1,n2,n3)
     */
    public static Number max(Number n1, Number n2, Number n3) {
	return max(max(n1, n2), n3);
    }
    
    /*
     * Returns a string representation of the object.
     * @return a string representation of the object
     */
    public String toString() {
	return new String(alap + "E" + exponent);
    }
    
    /*
     * Differencialfuggveny ket Number tipusu szam kozott
     * @return kulonbsegfuggveny erteke
     */
    public static double diff(Number n1, Number n2, double eps) {
	//		if (n1.exponent == n2.exponent && Math.sqrt(Math.pow(n1.alap - n2.alap,2)) < eps) {
	//			return 0.0;
	//		} else {
	return n1.exponent - n2.exponent + (n1.alap - n2.alap) / Integer.MAX_VALUE;
	//		}
    }
    
    public double toDouble() {
	return Math.pow(10, exponent) * alap;
    }
    
    public boolean equals(Number n) {
	if (alap == n.alap && n.alap == 0) {
	    return true;
	}
	if (alap == n.alap && exponent == n.exponent) {
	    return true;
	} else {
	    return false;
	}
    }
}
