import java.io.*;
import java.util.*;

public class PatternSet {
	
	public class Item {
		private String name, value;
		
		Item(String name, String value) {
			this.name = name;
			this.value = value;
		}
		
		public String getName() {
			return name;
		}
		
		public String getValue() {
			return value;
		}
	}
	
	private ArrayList patternlist;
	
	/**
	 * Konstruktor.
	 * @param filename a feldolgozando file neve
	 */
	public PatternSet(String filename) {
		patternlist = new ArrayList();
		try {
			BufferedReader infile = new BufferedReader( new FileReader(filename) );
			
			String str;
			String name = "";
			String value = "";
			while ((str = infile.readLine()) != null) {
				if (str.length() == 0) {
					continue;
				}
				if (str.charAt(0) == '>') {
					if (name.length() > 0 && value.length() > 0) {						
						patternlist.add( new Item(name, value) );
					}
					name = str.substring(1).trim();
					value = "";
				} else {
					value += str.trim().replaceAll("[^A-Za-z]*", "");
				}
			}
			if (name.length() > 0 && value.length() > 0) {
				patternlist.add( new Item(name, value) );
			}
			infile.close();
		} catch (Exception e) {
			System.err.println( e.getMessage() );
		}
	}
	
	/**
	 * Elem lekerdezese a listabol.
	 * @param index a lekerdezendo elem indexe
	 * @return az elem
	 */
	public Item elementAt(int index) {
		return (Item)patternlist.get(index);
	}
	
	/**
	 * A listaban levo mintak szamanak lekerdezese.
	 * @return a mintak darabszama
	 */
	public int getCount() {
		if (patternlist == null) {
			return -1;
		} else {
			return patternlist.size();
		}
	}
	
	/**
	 * Megadja, hogy ures-e a mintakat tarolo sor.
	 * @return ures-e a sor
	 */
	public boolean isEmpty() {
		return (patternlist.size() == 0);
	}
	
}
