import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * Preprocess ablak
 * @author Dombai Balazs
 */
public class PreprocessDialog extends JDialog implements ActionListener {
    
    public boolean ok_clicked = false;
//    public ArrayList[] results;
    public SimilarSequences[] result;
    
    private Settings settings;
    private PatternSet ps;
    
    public SimilarSequences getResult() {
	return result[comboBox.getSelectedIndex()];
    }
    
    private void calculate() {
	setEnabled(false);
	
	double nevezo = ps.getCount() * (ps.getCount() - 1) / 2;
	double szamlalo = 0;
	
	int[] tmp = new int[ps.getCount()];
	boolean[][] tmp2 = new boolean[ps.getCount()][ps.getCount()];
	
	for (int i = 0; i < ps.getCount(); i++) {
	    tmp[i] = 0;
	}
	
	for (int i = 0; i < ps.getCount(); i++) {
	    result[i] = new SimilarSequences(ps.elementAt(i));
	    for (int j = 0; j < i; j++) {
		GotohPenalty gp = new GotohPenalty( new Blosum() );
		gp.init(ps.elementAt(i).getValue(), ps.elementAt(j).getValue());
		gp.run();
		if (gp.findMax() > settings.getMinSimilarity()) {
		    tmp[i]++;
		    tmp[j]++;
		    tmp2[i][j] = true;
		    tmp2[j][i] = true;
		} else {
		    tmp2[i][j] = false;
		    tmp2[j][i] = false;
		}
		progressBar.setValue( (int)Math.floor(100 * szamlalo / nevezo) );
		szamlalo++;
	    }
	}
	
	for (int i = 0; i < ps.getCount(); i++) {
	    for (int j = 0; j < ps.getCount(); j++) {
		if (i != j && tmp2[i][j] && tmp[j] >= settings.getMinMatches()) {
		    result[i].add( ps.elementAt(j) );
		}
	    }
	}
	
	for (int i = 0; i < ps.getCount(); i++) {
	    comboBox.addItem( ps.elementAt(i).getName() + " (" + result[i].getSize() + ")");
	}
	
	progressBar.setValue(100);
	
	setEnabled(true);
	this.repaint();
    } // calculate()
    
    private void okBn_clicked() {
	ok_clicked = true;
	setVisible(false);
    } // okBn_clicked()
    
    private void cancelBn_clicked() {
	setVisible(false);
    } // cancelBn_clicked()
    
    private void comboBox_selectionChanged() {
	int index = comboBox.getSelectedIndex();
	DefaultListModel model = (DefaultListModel)list.getModel();
	model.clear();
	
	if (result[index].getSize() == 0) {
	    okBn.setEnabled(false);
	} else {
	    for (int i = 0; i < result[index].getSize(); i++) {
		model.addElement( result[index].nameAt(i) );
	    }
	    okBn.setEnabled(true);
	    list.setSelectedIndex(0);
	}
    } // comboBox_selectionChanged()
    
    public PreprocessDialog(JFrame owner, PatternSet ps, Settings settings) {
	super(owner);
	
	this.ps = ps;
	this.settings = settings;
//	results = new ArrayList[ps.getCount()];
	result = new SimilarSequences[ps.getCount()];
	
	c = getContentPane();
	
	setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
	setLayout(new BorderLayout());
	setLocation(getParent().getX()+80,getParent().getY()+60);
	setModal(true);
	setPreferredSize( new Dimension(500, 300) );
	setResizable(false);
	setTitle(WINDOW_TITLE);
	
	initComponents();
	
	(new Thread() { public void run() {	calculate(); };}).start();
    } // PreprocessDialog(owner, ps)
    
    public void initComponents() {
	progressBar = new JProgressBar(0,100);
	progressBar.setStringPainted(true);
	JPanel progressPanel = new JPanel();
	progressPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
	progressPanel.add( new JLabel("Progress:") );
	progressPanel.add(progressBar);
	
	comboBox = new JComboBox();
	comboBox.setEditable(false);
	comboBox.addItemListener(new ItemListener() { public void itemStateChanged(ItemEvent e) { comboBox_selectionChanged(); }; });
	JPanel comboPanel = new JPanel();
	comboPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
	comboPanel.add( new JLabel("Select gene:") );
	comboPanel.add(comboBox);
	
	JPanel topPanel = new JPanel();
	topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.Y_AXIS));
	topPanel.add(progressPanel);
	topPanel.add(comboPanel);
	c.add(topPanel, BorderLayout.NORTH);
//XXXXXX
	list = new JList(new DefaultListModel());
	((DefaultListModel)list.getModel()).addElement("");
	JPanel centerPanel = new JPanel();
	centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.PAGE_AXIS));
	centerPanel.add( new JLabel("Similar genes:") );
	centerPanel.add( new JScrollPane(list) );
	c.add(centerPanel, BorderLayout.CENTER);
//XXXXXX
	(okBn = new JButton("OK")).addActionListener(this);
	okBn.setMnemonic('O');
	(cancelBn = new JButton("Cancel")).addActionListener(this);
	cancelBn.setMnemonic('C');
	JPanel bottomPanel = new JPanel();
	bottomPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
	bottomPanel.add(okBn);
	bottomPanel.add(cancelBn);
	c.add(bottomPanel, BorderLayout.SOUTH);
	
	pack();
    } // initComponents()
    
    public void actionPerformed(ActionEvent evt) {
	if (evt.getSource() == okBn) {
	    okBn_clicked();
	} else if (evt.getSource() == cancelBn) {
	    cancelBn_clicked();
	}
	
    } // actionPerformed()
    
    static final String WINDOW_TITLE = "Preprocessing";
    
    private Container c;
    // components
    private JButton okBn, cancelBn;
    private JProgressBar progressBar;
    private JComboBox comboBox;
    private JList list;
}
