import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * Run ablak
 * @author Dombai Balazs
 */
public class RunDialog extends JDialog implements ActionListener {
    
    public boolean abort_clicked = false;
    public ArrayList[] results = null;
    private PatternSet ps = null;
    private ResultSet rs = null;
    private SimilarSequences simSeq = null;
    private int max_step, eps;
    private String parentTitle;
    
    private Thread calcThread;
    
    public ResultSet getResultSet() {
	return rs;
//	return viterbi.getResultSet();
    }
    
    private void appendMessage(String message) {
	resultArea.append(message);
	resultArea.select(resultArea.getDocument().getLength(), resultArea.getDocument().getLength());
    }
    
    private void calculate() {
	setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	okBn.setEnabled(false);
	abortBn.setEnabled(true);
	
	long startTime = System.currentTimeMillis();
	for (int j = 0; j < simSeq.getSize(); j++) {
	    String seq1 = simSeq.getValue();
	    String seq2 = simSeq.sequenceAt(j);
	    appendMessage("*** Sequence " + (j+1) + " of " + simSeq.getSize() + " (" + simSeq.nameAt(j) + ")...\n");
	    appendMessage("Calculating initial parameters...\n");
	    SmithWaterman sw = new SmithWaterman(seq1, seq2, new GotohPenalty(new Blosum()));
	    
	    ConjGrad cg = new ConjGrad(sw.getParams(), seq1, seq2, eps);
	    appendMessage("Optimizing the parameters...\n");
	    
	    for (int i = 0; i < max_step; i++) {
		if (cg.end()) {
		    appendMessage("- epsilon reached in " + i + " steps.\n");
		    break;
		}
		progressBar.setValue(i);
		((JFrame)getParent()).setTitle(i + "/" + max_step + ", " + (j+1) + " of " + simSeq.getSize() + " - " + parentTitle);
		appendMessage("- step " + (i + 1) + " of " + max_step + ", value = " + cg.getValue() + "...\n");
		cg.step();
	    }
	    
	    progressBar.setValue(max_step);
	    ((JFrame)getParent()).setTitle(max_step + "/" + max_step + ", " + simSeq.getSize() + " of " + simSeq.getSize() + " - " + parentTitle);
	    
	    appendMessage("Running the Viterbi algorithm...\n");
	    Viterbi viterbi = new Viterbi(cg.getOptimizedParams(), seq1, seq2);
	    appendMessage("Viterbi value: " + viterbi.getProbability() + "\n");
	    
	    int tmp = 0;
	    for (int i = 0; i < viterbi.getResultSet().getMaxPos() + 1; i++) {
		if (viterbi.getResultSet().getName(i).charAt(0) != '-') {
		    rs.setValue(tmp, (double)(rs.getValue(tmp) * j + viterbi.getResultSet().getValue(i)) / (j+1) );
		    tmp++;
		}
	    }

	}
	long endTime = System.currentTimeMillis();
	appendMessage("Finished, execution time: "+ (endTime-startTime) +" sec.");
	
	abortBn.setEnabled(false);
	okBn.setEnabled(true);
	setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
    } // calculate()
    
    private void okBn_clicked() {
	setVisible(false);
    } // okBn_clicked()
    
    private void minimizeBn_clicked() {
	((JFrame)this.getParent()).setState(JFrame.ICONIFIED);
    } // minimizeBn_clicked
    
    private void abortBn_clicked() {
	calcThread.interrupt();
	abort_clicked = true;
	setVisible(false);
    } // abortBn_clicked()
    
    public RunDialog(JFrame owner, Settings settings, SimilarSequences simSeq) {
	super(owner);
	
	parentTitle = owner.getTitle();
	
	max_step = settings.getMaxStep();
	eps = settings.getEps();
	
	this.simSeq = simSeq;
	
	rs = new ResultSet(simSeq.getValue().length());
	for (int i = 0; i < simSeq.getValue().length(); i++) {
	    rs.set(i, simSeq.getValue().charAt(i) + "", 0);
	}
	
	c = getContentPane();
	
	setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
	setLayout(new BorderLayout());
	setLocation(getParent().getX()+80,getParent().getY()+60);
	setModal(true);
	setPreferredSize( new Dimension(600, 400) );
	setResizable(false);
	setTitle(WINDOW_TITLE);
	
	initComponents();
	
	calcThread = new Thread() { public void run() {	calculate(); };};
	calcThread.start();
    } // PreprocessDialog(owner, ps)
    
    public void initComponents() {
	progressBar = new JProgressBar(0,max_step);
	progressBar.setStringPainted(true);
	JPanel progressPanel = new JPanel();
	progressPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
	progressPanel.add( new JLabel("Current sequence progress:") );
	progressPanel.add(progressBar);
	
	JPanel topPanel = new JPanel();
	topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.Y_AXIS));
	topPanel.add(progressPanel);
	c.add(topPanel, BorderLayout.NORTH);
//XXXXXX
	resultArea = new JTextArea();
	resultArea.setEditable(false);
	resultArea.setFont(new Font("MONOSPACED", Font.PLAIN, 12));
	JPanel centerPanel = new JPanel();
	centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.Y_AXIS));
	centerPanel.add( new JScrollPane(resultArea) );
	c.add(centerPanel, BorderLayout.CENTER);
//XXXXXX
	(okBn = new JButton("OK")).addActionListener(this);
	okBn.setMnemonic('O');
	(minimizeBn = new JButton("Minimize")).addActionListener(this);
	minimizeBn.setMnemonic('M');
	(abortBn = new JButton("Abort")).addActionListener(this);
	abortBn.setMnemonic('A');
	JPanel bottomPanel = new JPanel();
	bottomPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
	bottomPanel.add(okBn);
	bottomPanel.add(minimizeBn);
	bottomPanel.add(abortBn);
	c.add(bottomPanel, BorderLayout.SOUTH);
	
	pack();
    } // initComponents()
    
    public void actionPerformed(ActionEvent evt) {
	if (evt.getSource() == okBn) {
	    okBn_clicked();
	} else if (evt.getSource() == minimizeBn) {
	    minimizeBn_clicked();
	} else if (evt.getSource() == abortBn) {
	    abortBn_clicked();
	}
	
    } // actionPerformed()
    
    static final String WINDOW_TITLE = "Running";
    
    private Container c;
    // components
    private JButton okBn, abortBn, minimizeBn;
    private JProgressBar progressBar;
    private JTextArea resultArea;
}
