import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Beallitasok ablak
 * @author Dombai Balazs
 */
public class SettingsDialog extends JDialog implements ActionListener {
	
	public boolean ok_clicked = false;
	
	private PatternSet ps;
	private Settings settings;
	
	public Settings getSettings() {
	    Settings s = new Settings();
	    s.setEps(epsilonSlider.getValue());
	    s.setMaxStep(Integer.parseInt(stepSpinner.getValue().toString()));
	    s.setMinSimilarity(Integer.parseInt(minimumSimilaritySpinner.getValue().toString()));
	    s.setMinMatches(Integer.parseInt(minimumMatchSpinner.getValue().toString()));
	    return s;
	}
	
	private void okBn_clicked() {
		ok_clicked = true;
		setVisible(false);
	}
	
	private void cancelBn_clicked() {
		setVisible(false);
	} // cancelBn_clicked()
	
	public SettingsDialog(JFrame owner, Settings settings) {
		super(owner);
		c = getContentPane();
		
		this.settings = new Settings(settings);
		
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));
		setLocation(getParent().getX()+80,getParent().getY()+60);
		setModal(true);
		//setPreferredSize( new Dimension(300, 200) );
		setResizable(false);
		setTitle(WINDOW_TITLE);
		
		initComponents();
	}
	
	public void initComponents() {		
		JPanel minimumSimilarityPanel = new JPanel();
		minimumSimilarityPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		minimumSimilarityPanel.add(new JLabel("Minimum similarity:"));
		minimumSimilaritySpinner = new JSpinner( new SpinnerNumberModel( settings.getMinSimilarity(), 1, 65535, 1) );
		minimumSimilarityPanel.add(minimumSimilaritySpinner);
		c.add(minimumSimilarityPanel);
		
		JPanel minimumMatchPanel = new JPanel();
		minimumMatchPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		minimumMatchPanel.add(new JLabel("Minimum match to other sequences:"));
		minimumMatchSpinner = new JSpinner( new SpinnerNumberModel( settings.getMinMatches(), 0, 65535, 1) );
		minimumMatchPanel.add(minimumMatchSpinner);
		c.add(minimumMatchPanel);
		
		JPanel stepPanel = new JPanel();
		stepPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		stepPanel.add(new JLabel("Max number of steps in maximizing (10-200):"));
		stepSpinner = new JSpinner( new SpinnerNumberModel( settings.getMaxStep(), 10, 200, 1) );
		stepPanel.add(stepSpinner);
		c.add(stepPanel);
		
		JPanel epsilonPanel = new JPanel();
		epsilonPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		epsilonPanel.add(new JLabel("Precision (decimal places):"));		
		epsilonSlider = new JSlider(2,10,settings.getEps());
		epsilonSlider.setMajorTickSpacing(1);
		epsilonSlider.setSnapToTicks(true);
		epsilonSlider.setPaintLabels(true);
		epsilonSlider.setPaintTicks(true);
		epsilonPanel.add(epsilonSlider);
		c.add(epsilonPanel);
		
		(okBn = new JButton("OK")).addActionListener(this);
		okBn.setMnemonic('O');
		(cancelBn = new JButton("Cancel")).addActionListener(this);
		cancelBn.setMnemonic('C');
		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		bottomPanel.add(okBn);
		bottomPanel.add(cancelBn);
		c.add(bottomPanel);
		
		pack();
	}
	
	public void actionPerformed(ActionEvent evt) {
		if (evt.getSource() == okBn) {
			okBn_clicked();
		} else if (evt.getSource() == cancelBn) {
			cancelBn_clicked();
		}
	}
	
	static final String WINDOW_TITLE = "Settings";
	
	private Container c;
	// components
	private JButton okBn, cancelBn;
	private JSpinner stepSpinner, minimumSimilaritySpinner, minimumMatchSpinner;
	private JSlider epsilonSlider;

}
