/**
 * Viterbi algoritmus implementacioja
 * @author Dombai Balazs
 */
public class Viterbi {
    
    private Hmm hmm;
    private Number[][][] matrix;
    private int[][][] d2;
    private ReticularAlignment ra;
    private ResultSet rs;
    
    private String seq1, seq2;
    
    public String aligned_seq1 = "";
    public String aligned_seq2 = "";
    
    private Forward f;
    private Backward b;
    
    private Number value;
    
    /**
     * Konstruktor.
     * @param params (t/r/mu/lambda) parameterek
     * @param seq1 az elso szekvencia
     * @param seq2 a masodik szekvencia
     */
    public Viterbi(Params params, String seq1, String seq2) {
	f = new Forward(params, seq1, seq2);
	b = new Backward(params, seq1, seq2);
	this.value = f.getProbability();
/*
	System.out.println("* Parameters(t/r/mu/lambda): " + params);
	System.out.println("* Forward probability: " + f.getProbability());
	System.out.println("* Backward probability: " + b.getProbability());
 */
	this.seq1 = '-' + seq1;
	this.seq2 = '-' + seq2;
	ra = new ReticularAlignment(params.get_t());
	//	params.set_r(0.01); // mimicking tkf91
	hmm = new Hmm(params);
	matrix = new Number[this.seq1.length()][this.seq2.length()][3];
	d2 = new int[this.seq1.length()][this.seq2.length()][3];
	
	run();
	backtrace();
    }
    
    /**
     * A teljes szekvencia emittalasi valoszinusegenek visszaadasa
     * @return valoszinuseg
     */
    public Number getProbability() {
	return Number.max(matrix[seq1.length()-1][seq2.length()-1][0],
	    matrix[seq1.length()-1][seq2.length()-1][1],
	    matrix[seq1.length()-1][seq2.length()-1][2]);
    }
    
    /**
     * A dinamikus programozasi algoritmus vegrehajtasa.
     */
    private void run() {
	
	matrix[0][0][0] = new Number(1);
	matrix[0][0][1] = new Number(0);
	matrix[0][0][2] = new Number(0);
	
	for (int j = 1; j < seq1.length(); j++) {
	    matrix[j][0][0] = new Number(0);
	    d2[j][0][0] = -1;
	    
	    matrix[j][0][1] = new Number(0);
	    Number temp = new Number(matrix[j-1][0][0]);
	    char c1 = seq1.charAt(j);
	    temp.mul(ra.getPi(c1) * hmm.valueAt(0,1));
	    matrix[j][0][1].add(temp);
	    temp = new Number(matrix[j-1][0][1]);
	    temp.mul(ra.getPi(c1) * hmm.valueAt(1,1));
	    d2[j][0][1] = Number.greater(matrix[j][0][1],temp) ? 0 : 1;
	    matrix[j][0][1] = Number.max(matrix[j][0][1],temp);
	    
	    matrix[j][0][2] = new Number(0);
	    d2[j][0][2] = -1;
	}
	for (int k = 1; k < seq2.length(); k++) {
	    matrix[0][k][0] = new Number(0);
	    d2[0][k][0] = -1;
	    matrix[0][k][1] = new Number(0);
	    d2[0][k][1] = -1;
	    
	    matrix[0][k][2] = new Number(0);
	    Number temp = new Number(matrix[0][k-1][0]);
	    char c2 = seq2.charAt(k);
	    temp.mul(ra.getPi(c2) * hmm.valueAt(0,2));
	    matrix[0][k][2].add(temp);
	    temp = new Number(matrix[0][k-1][2]);
	    temp.mul(ra.getPi(c2) * hmm.valueAt(2,1));
	    d2[0][k][2] = Number.greater(matrix[0][k][2],temp) ? 0 : 1;
	    matrix[0][k][2] = Number.max(matrix[0][k][2],temp);
	    
	}
	
	
	
	for (int j = 1; j < seq1.length(); j++) {
	    for (int k = 1; k < seq2.length(); k++) {
		matrix[j][k][0] = new Number(0);
		matrix[j][k][1] = new Number(0);
		matrix[j][k][2] = new Number(0);
		
		char c1 = seq1.charAt(j);
		char c2 = seq2.charAt(k);
		int dir = -1;
		
		for (int i = 0; i < 3; i++) {
		    Number tmp = new Number(matrix[j-1][k-1][i]);
		    tmp.mul( ra.getPi(c1) * ra.getTrans(c2, c1) * hmm.valueAt(i,0));
		    if (Number.greater(tmp, matrix[j][k][0])) {
			dir = i;
		    }
		    matrix[j][k][0]  = Number.max(matrix[j][k][0], tmp);
		}
		d2[j][k][0] = dir;
		
		for (int i = 0; i < 3; i++) {
		    Number tmp = new Number(matrix[j-1][k][i]);
		    tmp.mul( ra.getPi(c1) * hmm.valueAt(i,1) );
		    if (Number.greater(tmp, matrix[j][k][1])) {
			dir = i;
		    }
		    matrix[j][k][1]  = Number.max(matrix[j][k][1], tmp);
		}
		d2[j][k][1] = dir;
		
		for (int i = 0; i < 3; i++) {
		    Number tmp = new Number(matrix[j][k-1][i]);
		    tmp.mul(  ra.getPi(c2) * hmm.valueAt(i,2) );
		    if (Number.greater(tmp, matrix[j][k][2])) {
			dir = i;
		    }
		    matrix[j][k][2]  = Number.max(matrix[j][k][2], tmp);
		}
		d2[j][k][2] = dir;
		
	    }
	}
	
    }
    
    public void backtrace() {
	
//	System.out.println("Traceback!!!\n************\n\n");
	int i = -1, j = seq1.length()-1, k = seq2.length()-1;
	double[] values = new double[j + k];
	int ptr = 0;
	
	Number max = Number.max(matrix[j][k][0],
	    matrix[j][k][1],
	    matrix[j][k][2]);
	
	if (Number.equals(max, matrix[j][k][0])) {
	    i = 0;
	} else if (Number.equals(max, matrix[j][k][1])) {
	    i = 1;
	} else if (Number.equals(max, matrix[j][k][2])) {
	    i = 2;
	}
	
	while (j > 0 && k > 0 && i > -1) {
	    
	    Number tmp = f.getPr(j,k,i);
	    tmp.mul(b.getPr(j,k,i));
	    tmp.alap /= value.alap;
	    tmp.exponent -= value.exponent;
	    tmp.optimize();
	    
	    values[ptr] = tmp.toDouble();
	    ptr++;
	    //eddig
	    
	    int i1 = d2[j][k][i];
	    if (i == 0) {
		aligned_seq1 = seq1.charAt(j) + aligned_seq1;
		aligned_seq2 = seq2.charAt(k) + aligned_seq2;
		j--;
		k--;
	    } else if (i == 1) {
		aligned_seq1 = seq1.charAt(j) + aligned_seq1;
		aligned_seq2 = '-' + aligned_seq2;
		j--;
	    } else if (i == 2) {
		aligned_seq1 = '-' + aligned_seq1;
		aligned_seq2 = seq2.charAt(k) + aligned_seq2;
		k--;
	    }
	    if (i < 0) {
		throw new Error("Negativ i!!!!");
	    }
	    i = i1;
	}
	
//	System.out.println(aligned_seq1+"\n"+aligned_seq2);
	
	rs = new ResultSet(aligned_seq1.length());
	for (int l = 0; l < aligned_seq1.length(); l++) {
	    rs.set(l, aligned_seq1.charAt(l) + "" + aligned_seq2.charAt(l), values[ptr - l - 1]);
	}
    }
    
    public ResultSet getResultSet() {
	return rs;
    }
    
}
